package com.jamieswhiteshirt.reachentityattributes;

import net.fabricmc.api.ModInitializer;
import net.minecraft.class_1297;
import net.minecraft.class_1309;
import net.minecraft.class_1320;
import net.minecraft.class_1324;
import net.minecraft.class_1329;
import net.minecraft.class_1657;
import net.minecraft.class_1937;
import net.minecraft.class_2378;
import net.minecraft.class_2960;
import net.minecraft.class_7923;
import javax.annotation.Nullable;
import javax.annotation.ParametersAreNonnullByDefault;
import D;
import java.util.ArrayList;
import java.util.List;
import java.util.function.Predicate;

@ParametersAreNonnullByDefault
public final class ReachEntityAttributes implements ModInitializer {
    public static final String MOD_ID = "reach-entity-attributes";

    public static final class_1320 REACH = make("reach", 0.0, -1024.0, 1024.0);
    public static final class_1320 ATTACK_RANGE = make("attack_range", 0.0, -1024.0, 1024.0);

    public static double getReachDistance(final class_1309 entity, final double baseReachDistance) {
        @Nullable final var reachDistance = entity.method_5996(REACH);
        return (reachDistance != null) ? (baseReachDistance + reachDistance.method_6194()) : baseReachDistance;
    }

    public static double getSquaredReachDistance(final class_1309 entity, final double sqBaseReachDistance) {
        final var reachDistance = getReachDistance(entity, Math.sqrt(sqBaseReachDistance));
        return reachDistance * reachDistance;
    }

    public static double getAttackRange(final class_1309 entity, final double baseAttackRange) {
        @Nullable final var attackRange = entity.method_5996(ATTACK_RANGE);
        return (attackRange != null) ? (baseAttackRange + attackRange.method_6194()) : baseAttackRange;
    }

    public static double getSquaredAttackRange(final class_1309 entity, final double sqBaseAttackRange) {
        final var attackRange = getAttackRange(entity, Math.sqrt(sqBaseAttackRange));
        return attackRange * attackRange;
    }

    public static List<class_1657> getPlayersWithinReach(final class_1937 world, final int x, final int y, final int z, final double baseReachDistance) {
        return getPlayersWithinReach(player -> true, world, x, y, z, baseReachDistance);
    }

    public static List<class_1657> getPlayersWithinReach(final Predicate<class_1657> viewerPredicate, final class_1937 world, final int x, final int y, final int z, final double baseReachDistance) {
        final List<class_1657> playersWithinReach = new ArrayList<>(0);
        for (final class_1657 player : world.method_18456()) {
            if (viewerPredicate.test(player)) {
                final var reach = getReachDistance(player, baseReachDistance);
                final var dx = (x + 0.5) - player.method_23317();
                final var dy = (y + 0.5) - player.method_23320();
                final var dz = (z + 0.5) - player.method_23321();
                if (((dx * dx) + (dy * dy) + (dz * dz)) <= (reach * reach)) {
                    playersWithinReach.add(player);
                }
            }
        }
        return playersWithinReach;
    }

    public static boolean isWithinAttackRange(final class_1657 player, final class_1297 entity) {
        return player.method_5858(entity) <= getSquaredAttackRange(player, 64.0);
    }

    private static class_1320 make(final String name, final double base, final double min, final double max) {
        return new class_1329("attribute.name.generic." + MOD_ID + '.' + name, base, min, max).method_26829(true);
    }

    @Override
    public void onInitialize() {
        class_2378.method_10230(class_7923.field_41190, new class_2960(MOD_ID, "reach"), REACH);
        class_2378.method_10230(class_7923.field_41190, new class_2960(MOD_ID, "attack_range"), ATTACK_RANGE);
    }
}
